import numpy as np
import pandas as pd
from ovretl.containers_utils.merge_shipments_with_containers import (
    split_propositions_shipments_containers,
    merge_with_shipments_containers_then_with_propositions_containers,
)
from pandas.util.testing import assert_frame_equal


def test_merge_with_shipments_containers_then_with_propositions_containers():
    containers_df = pd.DataFrame(
        data={
            "teus": [4, 1, 2, 2],
            "tc": [1, 1, 1, 1],
            "shipment_id": [np.nan, np.nan, "2", "3"],
            "proposition_id": ["0", "1", np.nan, np.nan],
            "hazardous": [True, True, False, False],
            "refrigerated": [False, False, True, True],
        }
    )
    shipments_df = pd.DataFrame(data={"shipment_id": [np.nan, "2", "3"], "proposition_id": ["0", "1", np.nan]})
    shipments_df_true = pd.DataFrame(
        data={
            "shipment_id": ["2", "3", np.nan],
            "proposition_id": ["1", np.nan, "0"],
            "teus": [2.0, 2.0, 4.0],
            "tc": [1.0, 1.0, 1.0],
            "containers_hazardous": [False, False, True],
            "containers_refrigerated": [True, True, False],
        }
    )
    (propositions_containers_df, shipments_containers_df,) = split_propositions_shipments_containers(containers_df)

    shipments_df_processed = merge_with_shipments_containers_then_with_propositions_containers(
        shipments_df=shipments_df,
        shipments_containers_df=shipments_containers_df,
        propositions_containers_df=propositions_containers_df,
    )
    assert_frame_equal(shipments_df_processed, shipments_df_true, check_dtype=False)



import pandas as pd
from ovretl.transit_times.add_features import add_features
from ovretl.transit_times.constants import DEFAULT_ENCODING
from ovretl.transit_times.encode_features import encode_features


def preprocess_transit_times(transit_times_df: pd.DataFrame, outlier_threshold: int):
    transit_times_df = transit_times_df[transit_times_df["freight_method"].isin(["air", "ocean"])]
    transit_times_df = transit_times_df[
        ~(transit_times_df["pickup_time"].isnull()) | ~(transit_times_df["delivery_time"].isnull())
    ]
    transit_times_df = transit_times_df[
        (transit_times_df["pickup_time"].isnull())
        | (transit_times_df["pickup_time"].apply(lambda x: x < outlier_threshold))
    ]
    transit_times_df = transit_times_df[
        (transit_times_df["delivery_time"].isnull())
        | (transit_times_df["delivery_time"].apply(lambda x: x < outlier_threshold))
    ]
    transit_times_df = encode_features(transit_times_df)
    transit_times_df = add_features(transit_times_df)
    transit_times_df.loc[:, "distance"] = transit_times_df["distance"].fillna(0)
    transit_times_df = transit_times_df.fillna(DEFAULT_ENCODING)
    transit_times_df = transit_times_df.sort_values(by="shipment_created_at")
    return transit_times_df



import pandas as pd


def merge_all_features(
    shipments_df: pd.DataFrame,
    prices_origin_df: pd.DataFrame,
    turnover_df: pd.DataFrame,
    vat_df: pd.DataFrame,
    margin_without_insurance_df: pd.DataFrame,
    margin_insurance_df: pd.DataFrame,
    category_all_prices_df: pd.DataFrame,
    category_main_prices_df: pd.DataFrame,
) -> pd.DataFrame:
    shipments_df = pd.merge(
        shipments_df, prices_origin_df[["shipment_id", "prices_origin"]], on="shipment_id", how="left",
    )

    shipments_df = pd.merge(
        shipments_df, turnover_df, on="shipment_id", how="left",
    )

    shipments_df = pd.merge(shipments_df, vat_df[["shipment_id", "vat_price"]], on="shipment_id", how="left",)

    shipments_df = pd.merge(
        shipments_df,
        margin_without_insurance_df[["shipment_id", "margin_without_insurance", "initial_margin_without_insurance",]],
        on="shipment_id",
        how="left",
    )

    shipments_df = pd.merge(
        shipments_df,
        margin_insurance_df[["shipment_id", "margin_insurance", "initial_margin_insurance"]],
        on="shipment_id",
        how="left",
    )

    final = pd.merge(shipments_df, category_all_prices_df, on="shipment_id", how="left",)

    return pd.merge(final, category_main_prices_df, on="shipment_id", how="left",)



import pandas as pd


def merge_with_shipments_loads_then_with_propositions_loads(
        shipments_df: pd.DataFrame, shipments_loads_df: pd.DataFrame, propositions_loads_df: pd.DataFrame,
) -> pd.DataFrame:
    """
    The former logic implemented used to first merge with propositions loads then with shipments loads. Now it is the
    contrary. We still don't know when the logic has been changed on Tech side, but the Data did this fix on Monday,
    July 18th 2022.
    NB: Most of the time, the shipment's loads info and the shipment's best proposition's loads info are the same, but
    may differ in case an operational has not rigorously updated the proposition after editing the shipment's loads.
    Hence, it is better to rely primarily on the shipment's loads info.
    :param shipments_df:
    :param shipments_loads_df:
    :param propositions_loads_df:
    :return:
    Note:
        Modified date: 18-07-2022
        Author: SB
    """
    shipments_df = pd.merge(
        shipments_df,
        shipments_loads_df[
            [
                "total_number",
                "total_volume_in_cbm",
                "total_volume_in_cbf",
                "total_weight_in_kg",
                "total_weight_in_lbs",
                "taxable_weight_in_kg",
                "weight_measurable",
                "hazardous",
                "lithium",
                "non_stackable",
                "shipment_id",
            ]
        ],
        how="left",
        on="shipment_id",
    )

    shipments_df_with_total_weight = shipments_df[~shipments_df["total_weight_in_kg"].isnull()]
    shipments_df_without_total_weight = shipments_df[shipments_df["total_weight_in_kg"].isnull()].drop(
        [
            "total_number",
            "total_weight_in_kg",
            "total_weight_in_lbs",
            "total_volume_in_cbm",
            "total_volume_in_cbf",
            "taxable_weight_in_kg",
            "weight_measurable",
            "hazardous",
            "lithium",
            "non_stackable",
        ],
        axis=1,
    )

    shipments_df_without_total_weight = pd.merge(
        shipments_df_without_total_weight,
        propositions_loads_df[
            [
                "total_number",
                "total_volume_in_cbm",
                "total_volume_in_cbf",
                "total_weight_in_kg",
                "total_weight_in_lbs",
                "taxable_weight_in_kg",
                "weight_measurable",
                "hazardous",
                "magnetic",
                "lithium",
                "refrigerated",
                "non_stackable",
                "proposition_id",
            ]
        ],
        how="left",
        on="proposition_id",
    )

    return (
        pd.concat([shipments_df_with_total_weight, shipments_df_without_total_weight], sort=False,)
        .reset_index(drop=True)
        .rename(
            columns={
                "hazardous": "loads_hazardous",
                "magnetic": "loads_magnetic",
                "lithium": "loads_lithium",
                "refrigerated": "loads_refrigerated",
            }
        )
    )


def split_propositions_shipments_loads(loads_df: pd.DataFrame):
    propositions_loads_df = loads_df[~loads_df["proposition_id"].isnull()]
    shipments_loads_df = loads_df[~loads_df["shipment_id"].isnull()]
    return propositions_loads_df, shipments_loads_df


def merge_shipments_with_loads(shipments_df: pd.DataFrame, loads_df: pd.DataFrame, drop_duplicate_key="shipment_id"):
    (propositions_loads_df, shipments_loads_df,) = split_propositions_shipments_loads(loads_df)
    shipments_df = merge_with_shipments_loads_then_with_propositions_loads(
        shipments_df=shipments_df, shipments_loads_df=shipments_loads_df, propositions_loads_df=propositions_loads_df,
    )
    shipments_df = shipments_df.sort_values(by=["total_weight_in_kg", "total_volume_in_cbm"], ascending=False)
    shipments_df = shipments_df[
        (~shipments_df.duplicated(subset=[drop_duplicate_key])) | (shipments_df[drop_duplicate_key].isnull())
    ]  # drop duplicates without dropping nan
    return shipments_df



import pandas as pd


def drop_not_arrived_shipment_groups(shipment_events_df: pd.DataFrame):
    shipment_events_df = shipment_events_df.sort_values(by=["date"], ascending=False).reset_index(drop=True)
    if shipment_events_df.loc[0, "date_type"] == "estimated":
        return False
    return True


def extract_arrived_date_from_events(events_shipment_df: pd.DataFrame):
    events_shipment_df = events_shipment_df.dropna(subset=["shipment_id", "date"], how="any").drop(
        ["created_at"], axis=1
    )
    events_shipment_df = events_shipment_df.groupby("shipment_id").filter(drop_not_arrived_shipment_groups)
    events_shipment_df = events_shipment_df.sort_values(by=["shipment_id", "date"], ascending=False)
    events_shipment_df = events_shipment_df.drop_duplicates(subset=["shipment_id"])
    events_shipment_df.loc[:, "header"] = "shipment_arrived"
    events_shipment_df = events_shipment_df.rename(columns={"date": "created_at"})
    return events_shipment_df[["created_at", "header", "shipment_id"]]


def extract_finished_date_from_shipments(shipments_df: pd.DataFrame):
    shipments_df = shipments_df.dropna(subset=["finished_date"]).drop(["created_at"], axis=1)
    shipments_df.loc[:, "header"] = "shipment_finished"
    shipments_df = shipments_df.rename(columns={"finished_date": "created_at", "id": "shipment_id"})
    return shipments_df[["created_at", "header", "shipment_id"]]


def concatenate_activities(
    events_shipment_df: pd.DataFrame, activities_clean_df: pd.DataFrame, shipments_df: pd.DataFrame,
):
    activities_clean_df = pd.concat([activities_clean_df, events_shipment_df, shipments_df], sort=False)
    activities_clean_df.loc[:, "created_at"] = pd.to_datetime(activities_clean_df["created_at"]).apply(
        lambda t: t.replace(tzinfo=None)
    )
    activities_clean_df = activities_clean_df.sort_values(by=["shipment_id", "header", "created_at"], ascending=True)
    activities_clean_df = activities_clean_df.drop_duplicates(subset=["shipment_id", "header"])

    activities_clean_df = activities_clean_df[
        activities_clean_df["header"].isin(
            [
                'quotation_asked',
                'request_considered',
                'operations_owner_assigned',
                'quotation_purchase_ready',
                'propositions_received',
                'quotation_accepted',
                'shipment_booked',
                'shipment_arrived',
                'shipment_finished',
                'billing_available',
                'billing_paid',
            ]
        )
    ]
    activities_clean_df = pd.pivot_table(
        activities_clean_df, values="created_at", index=["shipment_id"], columns=["header"], aggfunc="first",
    )
    return activities_clean_df.reset_index()


def calculate_shipments_treatment_steps(
    events_shipment_df: pd.DataFrame, activities_clean_df: pd.DataFrame, shipments_df: pd.DataFrame,
):
    events_shipment_df = extract_arrived_date_from_events(events_shipment_df=events_shipment_df)
    activities_clean_df = activities_clean_df.dropna(subset=["shipment_id"])[
        ["created_at", "header", "shipment_id", "employee_id"]
    ]
    shipments_df = extract_finished_date_from_shipments(shipments_df=shipments_df)

    activities_clean_df = concatenate_activities(
        events_shipment_df=events_shipment_df, activities_clean_df=activities_clean_df, shipments_df=shipments_df,
    )
    return activities_clean_df.drop(["employee_id"], axis=1, errors="ignore")



import numpy as np
import pandas as pd
from ovretl.tracking_utils.check_shipment_active import check_shipment_active
from ovretl.tracking_utils.check_shipment_arrived import check_shipment_arrived
from ovretl.tracking_utils.check_shipment_in_progress import check_shipment_in_progress


def extract_event_date_by_sorting(events_shipment_df: pd.DataFrame, ascending: bool, location_key=None):
    if len(events_shipment_df) == 0:
        return np.nan
    events_shipment = events_shipment_df.sort_values(by="date", ascending=ascending).reset_index(drop=True)
    if len(events_shipment) > 0 and location_key is not None:
        # In this script, the location_key variable takes only "harbor" and "warehouse" as values, whereas the
        # "location_type" fields of the "events" entity can take the following (NULL, "warehouse", "harbor_air",
        # "harbor_ocean", "harbor_rail"). That is why we use startswith instead of == (especially when "location_type"
        # starts with "harbor_". This logic could change if tomorrow a new migration happens on the "location_type"
        # field of the "events" entity. By the way, we add a check to ensure the "location_type" field is not None, in
        # order to use the startswith method on a string (and we add a return np.nan in case the "location_type" field
        # is not None but the startswith is not verified).
        if events_shipment.loc[0, "location_type"] is not None and events_shipment.loc[0, "location_type"].startswith(
                location_key):
            return (
                pd.to_datetime(events_shipment.loc[0, "date"])
                if not pd.isna(events_shipment.loc[0, "date"])
                else np.nan
            )
        return np.nan
    if len(events_shipment) > 0:
        return pd.to_datetime(events_shipment.loc[0, "date"]) if not pd.isna(events_shipment.loc[0, "date"]) else np.nan
    return np.nan


def extract_pickup_event_date(events_shipment_df: pd.DataFrame, date_type=None):
    mask_date_type = events_shipment_df["date_type"].apply(
        lambda event_date_type: event_date_type == date_type if date_type is not None else True
    )
    mask_etp = events_shipment_df["is_used_for_pickup"] == True
    mask_event_pickup = events_shipment_df["event_description"].apply(lambda e: "Pickup" in e)
    mask_event_not_delivery = events_shipment_df["event_description"].apply(lambda e: "Delivery" not in e)

    event_found_by_etp = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_etp & mask_date_type], ascending=True
    )
    event_found_by_description = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_pickup & mask_date_type], ascending=True,
    )
    event_found_by_order = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_not_delivery & mask_date_type],
        ascending=True,
        location_key="warehouse",
    )
    if not pd.isna(event_found_by_etp):
        return event_found_by_etp
    if not pd.isna(event_found_by_description):
        return event_found_by_description
    if not pd.isna(event_found_by_order):
        return event_found_by_order
    return np.nan


def extract_departure_event_date(events_shipment_df: pd.DataFrame, date_type=None):
    mask_date_type = events_shipment_df["date_type"].apply(
        lambda event_date_type: event_date_type == date_type if date_type is not None else True
    )
    mask_etd = events_shipment_df["is_used_for_etd"] == True
    mask_event_departure = events_shipment_df["event_description"].apply(lambda e: "Departure" in e)
    mask_event_not_arrival = events_shipment_df["event_description"].apply(lambda e: "Arrival" not in e)

    event_found_by_etd = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_etd & mask_date_type], ascending=True
    )
    event_found_by_description = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_departure & mask_date_type], ascending=True,
    )
    event_first = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_not_arrival & mask_date_type],
        ascending=True,
        location_key="harbor",
    )
    if not pd.isna(event_found_by_etd):
        return event_found_by_etd
    if not pd.isna(event_found_by_description):
        return event_found_by_description
    if not pd.isna(event_first):
        return event_first
    return np.nan


def extract_arrival_event_date(events_shipment_df: pd.DataFrame, date_type=None):
    mask_date_type = events_shipment_df["date_type"].apply(
        lambda event_date_type: event_date_type == date_type if date_type is not None else True
    )
    mask_eta = events_shipment_df["is_used_for_eta"] == True
    mask_event_arrival = events_shipment_df["event_description"].apply(lambda e: "Arrival" in e)
    mask_event_not_departure = events_shipment_df["event_description"].apply(lambda e: "Departure" not in e)

    event_found_by_eta = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_eta & mask_date_type], ascending=False,
    )
    event_found_by_description = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_arrival & mask_date_type], ascending=False,
    )
    event_last = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_not_departure & mask_date_type],
        ascending=False,
        location_key="harbor",
    )
    if not pd.isna(event_found_by_eta):
        return event_found_by_eta
    if not pd.isna(event_found_by_description):
        return event_found_by_description
    if not pd.isna(event_last):
        return event_last
    return np.nan


def extract_delivery_event_date(events_shipment_df: pd.DataFrame, date_type=None):
    mask_date_type = events_shipment_df["date_type"].apply(
        lambda event_date_type: event_date_type == date_type if date_type is not None else True
    )
    mask_etd = events_shipment_df["is_used_for_delivery"] == True
    mask_event_delivery = events_shipment_df["event_description"].apply(lambda e: "Delivery" in e)
    mask_event_not_pickup = events_shipment_df["event_description"].apply(lambda e: "Pickup" not in e)

    event_found_by_etd = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_etd & mask_date_type], ascending=False,
    )
    event_found_by_description = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_delivery & mask_date_type], ascending=False,
    )
    event_last = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_not_pickup & mask_date_type],
        ascending=False,
        location_key="warehouse",
    )
    if not pd.isna(event_found_by_etd):
        return event_found_by_etd
    if not pd.isna(event_found_by_description):
        return event_found_by_description
    if not pd.isna(event_last):
        return event_last
    return np.nan


def process_events(events_df: pd.DataFrame) -> pd.DataFrame:
    pickup_dates = events_df.groupby("shipment_id").apply(extract_pickup_event_date).rename("pickup_date")
    departure_dates = events_df.groupby("shipment_id").apply(extract_departure_event_date).rename("departure_date")
    arrival_dates = events_df.groupby("shipment_id").apply(extract_arrival_event_date).rename("arrival_date")
    delivery_dates = events_df.groupby("shipment_id").apply(extract_delivery_event_date).rename("delivery_date")
    pickup_dates_actual = (
        events_df.groupby("shipment_id")
        .apply(lambda e: extract_pickup_event_date(e, "actual"))
        .rename("actual_pickup_date")
    )
    departure_dates_actual = (
        events_df.groupby("shipment_id")
        .apply(lambda e: extract_departure_event_date(e, "actual"))
        .rename("actual_departure_date")
    )
    arrival_dates_actual = (
        events_df.groupby("shipment_id")
        .apply(lambda e: extract_arrival_event_date(e, "actual"))
        .rename("actual_arrival_date")
    )
    delivery_dates_actual = (
        events_df.groupby("shipment_id")
        .apply(lambda e: extract_delivery_event_date(e, "actual"))
        .rename("actual_delivery_date")
    )
    initial_eta_etd = events_df.drop_duplicates(subset=["shipment_id"])

    processed_dates = [
        pickup_dates,
        departure_dates,
        arrival_dates,
        delivery_dates,
        pickup_dates_actual,
        departure_dates_actual,
        arrival_dates_actual,
        delivery_dates_actual,
    ]
    events_processed_df = pd.concat(processed_dates, axis=1, keys=[s.name for s in processed_dates]).reset_index()
    return pd.merge(
        events_processed_df,
        initial_eta_etd[["shipment_id", "initial_eta", "initial_etd"]],
        on="shipment_id",
        how="left",
    )


def is_cancelled(shipment):
    if shipment["cancelled"]:
        # if it's a shipment
        if shipment["shipment_status"] in ["arrived", "awaiting_booking", "booked", "booking_request", "finished",
                                           "in_progress"]:
            shipment["shipment_status"] = "cancelled"
        # if it's a quotation
        elif shipment["shipment_status"] in ["accepted", "expired", "new", "not_answered", "propositions_sent",
                                             "purchase_ready", "to_be_requoted", "waiting_for_external"]:
            shipment["shipment_status"] = "archived_declined"
    return shipment


def add_tracking_event_dates(shipments_df: pd.DataFrame, events_shipment_df: pd.DataFrame) -> pd.DataFrame:
    events_processed_df = process_events(events_df=events_shipment_df)
    shipments_df = pd.merge(shipments_df, events_processed_df, how="left", on="shipment_id")
    shipments_df.loc[:, "is_active"] = shipments_df.apply(
        lambda s: check_shipment_active(s, events_shipment_df), axis=1
    )
    shipments_df.loc[:, "shipment_status"] = shipments_df.apply(
        lambda s: check_shipment_in_progress(s, events_shipment_df), axis=1
    )
    shipments_df.loc[:, "shipment_status"] = shipments_df.apply(
        lambda s: check_shipment_arrived(s, events_shipment_df), axis=1
    )
    shipments_df.loc[:, "shipment_status_before_potential_cancellation"] = shipments_df.loc[:, "shipment_status"]
    shipments_df.loc[:, "shipment_status"] = shipments_df.apply(
        lambda s: is_cancelled(s), axis=1)
    return shipments_df



import pandas as pd
from ovretl.prices_utils.add_category_prices_to_shipments import add_category_prices_to_shipments
from ovretl.prices_utils.add_prices_ids_to_shipments import add_prices_ids_to_shipments
from ovretl.prices_utils.features_functions.calculate_category_prices import (
    pivot_category_all_prices,
    pivot_category_main_prices,
)
from ovretl.prices_utils.features_functions.calculate_margins import (
    calculate_margin_without_insurance,
    calculate_margin_insurance,
)
from ovretl.prices_utils.features_functions.calculate_turnover import calculate_turnover
from ovretl.prices_utils.features_functions.calculate_vat import calculate_vat
from ovretl.prices_utils.features_functions.determine_prices_origin import determine_prices_origin
from ovretl.prices_utils.features_functions.merge_all_features import merge_all_features


columns_prices = [
    "initial_margin_without_insurance",
    "initial_margin_insurance",
    "margin_without_insurance",
    "margin_insurance",
    "turnover",
    "departure_truck_freight_price",
    "departure_truck_freight_main_price",
    "departure_fees_price",
    "freight_price",
    "arrival_fees_price",
    "arrival_truck_freight_price",
    "arrival_truck_freight_main_price",
    "customs_price",
    "insurance_price",
    "vat_price",
    "other_price",
    "departure_truck_freight_initial_price",
    "departure_truck_freight_initial_main_price",
    "departure_fees_initial_price",
    "freight_initial_price",
    "arrival_fees_initial_price",
    "arrival_truck_freight_initial_price",
    "arrival_truck_freight_initial_main_price",
    "insurance_initial_price",
    "departure_truck_freight_initial_purchase_price",
    "departure_truck_freight_initial_purchase_main_price",
    "departure_fees_initial_purchase_price",
    "freight_initial_purchase_price",
    "arrival_fees_initial_purchase_price",
    "arrival_truck_freight_initial_purchase_price",
    "arrival_truck_freight_initial_purchase_main_price",
    "insurance_initial_purchase_price",
    "departure_truck_freight_purchase_price",
    "departure_truck_freight_purchase_main_price",
    "departure_fees_purchase_price",
    "freight_purchase_price",
    "arrival_fees_purchase_price",
    "arrival_truck_freight_purchase_price",
    "arrival_truck_freight_purchase_main_price",
    "insurance_purchase_price",
]


def add_prices_to_shipments(
    shipments_df: pd.DataFrame,
    final_checks_df: pd.DataFrame,
    billings_df: pd.DataFrame,
    all_prices_final_check_by_category_df: pd.DataFrame,
    all_prices_billings_by_category_df: pd.DataFrame,
    all_prices_propositions_by_category_df: pd.DataFrame,
    main_prices_final_check_by_truck_freight_category_df: pd.DataFrame,
    main_prices_billings_by_truck_freight_category_df: pd.DataFrame,
    main_prices_propositions_by_truck_freight_category_df: pd.DataFrame,
) -> pd.DataFrame:
    shipments_with_prices_ids = add_prices_ids_to_shipments(
        shipments_df=shipments_df, final_checks_df=final_checks_df, billings_df=billings_df,
    )
    shipments_with_all_category_prices_df = add_category_prices_to_shipments(
        shipments_with_prices_ids,
        all_prices_final_check_by_category_df,
        all_prices_billings_by_category_df,
        all_prices_propositions_by_category_df,
    )
    shipments_with_main_category_prices_df = add_category_prices_to_shipments(
        shipments_with_prices_ids,
        main_prices_final_check_by_truck_freight_category_df,
        main_prices_billings_by_truck_freight_category_df,
        main_prices_propositions_by_truck_freight_category_df,
    )
    prices_origin_df = determine_prices_origin(shipments_with_all_category_prices_df)
    turnover_df = calculate_turnover(shipments_with_all_category_prices_df)
    vat_df = calculate_vat(shipments_with_all_category_prices_df)
    margin_without_insurance_df = calculate_margin_without_insurance(shipments_with_all_category_prices_df)
    margin_insurance_df = calculate_margin_insurance(shipments_with_all_category_prices_df)
    category_all_prices_df = pivot_category_all_prices(shipments_with_all_category_prices_df)
    category_main_prices_df = pivot_category_main_prices(shipments_with_main_category_prices_df)
    shipments_df = merge_all_features(
        shipments_df,
        prices_origin_df=prices_origin_df,
        turnover_df=turnover_df,
        vat_df=vat_df,
        margin_without_insurance_df=margin_without_insurance_df,
        margin_insurance_df=margin_insurance_df,
        category_all_prices_df=category_all_prices_df,
        category_main_prices_df=category_main_prices_df,
    )
    shipments_df.loc[:, columns_prices] = shipments_df[columns_prices].fillna(0)
    return shipments_df



import os

import pandas as pd
from pandas.util.testing import assert_frame_equal
from ovretl.performances_utils.calculate_shipments_treatment_steps import calculate_shipments_treatment_steps

cwd = os.path.dirname(__file__)


def test_calculate_shipments_treatment_steps():
    events_shipment_df = pd.read_csv(os.path.join(cwd, "./events_shipment.csv"))
    activities_clean_df = pd.read_csv(os.path.join(cwd, "./activities_clean.csv"))
    shipments_df = pd.read_csv(os.path.join(cwd, "./shipments_df.csv"))
    result = calculate_shipments_treatment_steps(
        events_shipment_df=events_shipment_df, activities_clean_df=activities_clean_df, shipments_df=shipments_df,
    )
    result_should_be = pd.read_csv(
        os.path.join(cwd, "shipments_treatment_steps.csv"), parse_dates=[i for i in range(1, 11)]
    )
    # to change this test by generating data inside of this script instead of using deprecated CSVs
    '''
    assert_frame_equal(
        result.sort_values("shipment_id").reset_index(drop=True).fillna(0),
        result_should_be.sort_values("shipment_id").reset_index(drop=True).fillna(0),
        check_dtype=False,
        check_column_type=False,
        check_less_precise=True,
        check_names=False,
        check_exact=False,
        check_datetimelike_compat=True,
    )
    '''
    return None



from ovretl.prices_utils.sum_multiple_billings_prices import sum_multiple_billings_prices
import pandas as pd


def add_initial_category_prices_to_shipments(
    shipments_with_prices: pd.DataFrame, prices_propositions_by_category_df: pd.DataFrame,
) -> pd.DataFrame:
    initial_prices = (
        prices_propositions_by_category_df.dropna(
            subset=["purchase_price_in_eur", "price_in_eur", "margin_price_in_eur"],
            how="all").rename(columns={"purchase_price_in_eur": "initial_purchase_price_in_eur",
                                       "price_in_eur": "initial_price_in_eur",
                                       "margin_price_in_eur": "initial_margin_price_in_eur",
                                       "proposition_id": "initial_proposition_id",
                                       }
                              )
    )

    shipments_with_prices = pd.merge(
        shipments_with_prices,
        initial_prices[
            [
                "initial_proposition_id",
                "category",
                "initial_margin_price_in_eur",
                "initial_price_in_eur",
                "initial_purchase_price_in_eur",
            ]
        ],
        on=("initial_proposition_id", "category"),
        how="left",
    )
    return shipments_with_prices


def add_category_prices_to_shipments(
    shipments_with_prices_ids: pd.Series,
    prices_final_check_by_category_df: pd.DataFrame,
    prices_billings_by_category_df: pd.DataFrame,
    prices_propositions_by_category_df: pd.DataFrame,
) -> pd.DataFrame:
    shipments_with_prices_billings_ids = shipments_with_prices_ids[[
        'shipment_id', 'foresea_name', 'billing_id']].drop_duplicates(subset=['billing_id'])
    shipments_with_prices_billings = pd.merge(
        shipments_with_prices_billings_ids,
        prices_billings_by_category_df.dropna(subset=["price_in_eur"], how="all"),
        on="billing_id",
        how="inner",
    )
    shipments_with_prices_final_check_ids = shipments_with_prices_ids[[
        'shipment_id', 'foresea_name', 'final_check_id', 'initial_proposition_id']].drop_duplicates(
        subset=['final_check_id'])
    shipments_with_prices_final_check = pd.merge(
        shipments_with_prices_final_check_ids,
        prices_final_check_by_category_df.dropna(subset=["margin_price_in_eur", "purchase_price_in_eur"], how="all"),
        on="final_check_id",
        how="inner",
    )
    shipments_with_prices = pd.concat([shipments_with_prices_billings, shipments_with_prices_final_check])
    if len(shipments_with_prices_final_check[(shipments_with_prices_final_check["margin_price_in_eur"].isnull()) | (
            shipments_with_prices_final_check["purchase_price_in_eur"].isnull())]) > 0:
        shipments_with_prices_propositions_ids = shipments_with_prices_ids[[
            'shipment_id', 'foresea_name', 'proposition_id']].drop_duplicates(subset=['proposition_id'])
        shipments_with_prices_propositions = pd.merge(
            shipments_with_prices_propositions_ids,
            prices_propositions_by_category_df[['category',
                                                'prices_origin',
                                                'proposition_id',
                                                'margin_price_in_eur',
                                                'purchase_price_in_eur']].dropna(
                subset=["margin_price_in_eur", "purchase_price_in_eur"], how="all"),
            on="proposition_id",
            how="inner",
        )
        shipments_with_prices = pd.concat([shipments_with_prices, shipments_with_prices_propositions])
    shipments_with_prices = add_initial_category_prices_to_shipments(
        shipments_with_prices=shipments_with_prices,
        prices_propositions_by_category_df=prices_propositions_by_category_df,
    )
    return sum_multiple_billings_prices(shipments_with_prices=shipments_with_prices)



import numpy as np
import pandas as pd

from ovretl import match_kronos_propositions_and_shipments


def test_match_kronos_propositions_and_shipments():
    kronos_propositions_df = pd.DataFrame(
        data={
            "id": [1, 2, 3, 4, 5, 6],
            "shipment_id": [0, 0, 1, 1, 2, 4],
            "proposition_id": [1, np.nan, 5, np.nan, np.nan, np.nan],
            "created_at": [1, 2, 3, 4, 6, 5],
        }
    )
    shipments_with_proposition_df = pd.DataFrame(
        data={
            "id": [1, 2, 3, 4, 10],
            "proposition_id": [1, 2, 3, 4, np.nan],
            "kronos_action": [np.nan, np.nan, "set_proposition_purchase_ready", "set_proposition_purchase_ready",
                              "set_proposition_purchase_ready"],
            "foresea_name": ["BATD", np.nan, "BATE", "BATF", "BATG"],
            "kronos_proposition_id": [np.nan, 100, 101, 6, 103],
        }
    )
    result_should_be = pd.DataFrame(
        data={
            "kronos_proposition_id": [6, 5, 3, 1],
            "shipment_id": [4, 2, 1, 0],
            "created_at": [5, 6, 3, 1],
            "proposition_id": [4, np.nan, 5, 1],
            "kronos_action": ["set_proposition_purchase_ready", np.nan, np.nan, np.nan],
        }
    )
    result = match_kronos_propositions_and_shipments(kronos_propositions_df, shipments_with_proposition_df)
    pd.testing.assert_frame_equal(result.reset_index(drop=True), result_should_be.reset_index(drop=True))



import pandas as pd
from pandas.util.testing import assert_frame_equal

from ovretl.performances_utils.calculate_number_purchase_readied import calculate_number_purchase_readied


def test_calculate_number_purchase_readied():
    activities_df = pd.DataFrame(
        data={
            "shipment_id": [0, 0, 1, 1],
            "employee_id": [0, 1, 2, 3],
            "header": ["quotation_purchase_ready", "quotation_purchase_ready", "quotation_purchase_ready", "bar"],
        }
    )
    shipments_df = pd.DataFrame(data={"shipment_id": [0, 1, 2]})
    result = calculate_number_purchase_readied(activities_df=activities_df, shipments_df=shipments_df)
    result_should_be = pd.DataFrame(data={"shipment_id": [0, 1, 2], "nb_purchase_readied": [2, 1, 0]})
    assert_frame_equal(result, result_should_be, check_dtype=False)



import numpy as np
import pandas as pd
from ovretl.loads_utils.merge_shipments_with_loads import (
    split_propositions_shipments_loads,
    merge_with_shipments_loads_then_with_propositions_loads,
)
from pandas.util.testing import assert_frame_equal

pd.set_option("display.max_columns", None)


def test_merge_with_shipments_loads_then_with_propositions_loads():
    loads_df = pd.DataFrame(
        data={
            "total_number": [1, 2, 1, 1],
            "total_weight_in_kg": [200, 100, 200, 200],
            "total_volume_in_cbm": [0.9, 0.9, 1, 0.96],
            "shipment_id": [np.nan, np.nan, "2", "3"],
            "proposition_id": ["0", "1", np.nan, np.nan],
            "taxable_weight_in_kg": [200, 150, 200, 200],
            "weight_measurable": [1, 1, 1, 1],
            "hazardous": [True, True, False, False],
            "non_stackable": [True, False, False, False],
            "magnetic": [False, False, False, False],
            "refrigerated": [False, False, False, False],
            "lithium": [False, False, False, False],
        }
    )
    shipments_df = pd.DataFrame(data={"shipment_id": [np.nan, "2", "3"], "proposition_id": ["0", "1", np.nan]})
    result_should_be = pd.DataFrame(
        data={
            "shipment_id": ["2", "3", np.nan],
            "proposition_id": ["1", np.nan, "0"],
            "total_number": [1.0, 1.0, 1.0],
            "total_volume_in_cbm": [1.00, 0.96, 0.90],
            "total_weight_in_kg": [200.0, 200.0, 200.0],
            "taxable_weight_in_kg": [200.0, 200.0, 200.0],
            "weight_measurable": [1.0, 1.0, 1.0],
            "loads_hazardous": [False, False, True],
            "loads_lithium": [False, False, False],
            "non_stackable": [False, False, True],
            "loads_magnetic": [np.nan, np.nan, False],
            "loads_refrigerated": [np.nan, np.nan, False],
        }
    )
    (propositions_loads_df, shipments_loads_df,) = split_propositions_shipments_loads(loads_df)

    result = merge_with_shipments_loads_then_with_propositions_loads(
        shipments_df=shipments_df, shipments_loads_df=shipments_loads_df, propositions_loads_df=propositions_loads_df,
    )
    assert_frame_equal(result, result_should_be, check_dtype=False)



import numpy as np
import pandas as pd
from ovretl.containers_utils.calculate_entity_containers import calculate_shipments_propositions_containers
from pandas.util.testing import assert_frame_equal


def test_calculate_entity_containers():
    containers_df = pd.DataFrame(
        data={
            "container_type": [
                "twenty_foot_reefer",
                "forty_foot_standard",
                "twenty_foot_standard",
                "forty_five_foot_highcube_standard",
                "forty_five_foot_highcube_standard",
            ],
            "hazardous": [True, True, False, False, False],
            "proposition_id": ["0", "0", "1", np.nan, np.nan],
            "shipment_id": [np.nan, np.nan, np.nan, "2", "3"],
        }
    )
    result_should_be = pd.DataFrame(
        data={
            "teus": [3, 1, 2, 2],
            "tc": [2, 1, 1, 1],
            "hazardous": [True, False, False, False],
            "refrigerated": [True, False, False, False],
            "proposition_id": ["0", "1", np.nan, np.nan],
            "shipment_id": [np.nan, np.nan, "2", "3"],
        }
    )
    result = calculate_shipments_propositions_containers(containers_df).reset_index(drop=True)
    assert_frame_equal(result, result_should_be, check_dtype=False, check_like=True)



import pandas as pd

from pandas.util.testing import assert_frame_equal
from ovretl.containers_utils.calculate_single_container_teus import calculate_single_container_teus


def test_calculate_single_container_teus():
    containers_df = pd.DataFrame(data={"container_type": ["twenty_foot_standard", "forty_five_foot_highcube_standard"]})
    containers_df_with_teus = pd.DataFrame(
        data={"container_type": ["twenty_foot_standard", "forty_five_foot_highcube_standard"], "teus": [1, 2],}
    )
    containers_df = containers_df.apply(calculate_single_container_teus, axis=1)
    assert_frame_equal(containers_df, containers_df_with_teus, check_dtype=False)



import pandas as pd


def merge_with_shipments_containers_then_with_propositions_containers(
        shipments_df: pd.DataFrame, shipments_containers_df: pd.DataFrame, propositions_containers_df: pd.DataFrame,
) -> pd.DataFrame:
    """
    The former logic implemented used to first merge with propositions containers then with shipments containers.
    Now it is the contrary. We still don't know when the logic has been changed on Tech side, but the Data did this fix
    on Monday, July 18th 2022.
    NB: Most of the time, the shipment's containers info and the shipment's best proposition's containers info are the
    same, but may differ in case an operational has not rigorously updated the proposition after editing the shipment's
    containers. Hence, it is better to rely primarily on the shipment's containers info.
    :param shipments_df:
    :param shipments_containers_df:
    :param propositions_containers_df:
    :return:
    Note:
        Modified date: 18-07-2022
        Author: SB
    """
    shipments_df = pd.merge(
        shipments_df,
        shipments_containers_df[["teus", "tc", "hazardous", "refrigerated", "shipment_id"]],
        how="left",
        on="shipment_id",
    )

    shipments_df_with_teus = shipments_df[~shipments_df["teus"].isnull()]
    shipments_df_without_teus = shipments_df[shipments_df["teus"].isnull()].drop(
        ["teus", "tc", "hazardous", "refrigerated"], axis=1
    )

    shipments_df_without_teus = pd.merge(
        shipments_df_without_teus,
        propositions_containers_df[["teus", "tc", "hazardous", "refrigerated", "proposition_id"]],
        how="left",
        on="proposition_id",
    )
    return (
        pd.concat([shipments_df_with_teus, shipments_df_without_teus], sort=False)
        .reset_index(drop=True)
        .rename(columns={"hazardous": "containers_hazardous", "refrigerated": "containers_refrigerated",})
    )


def split_propositions_shipments_containers(containers_df: pd.DataFrame):
    propositions_containers_df = containers_df[~containers_df["proposition_id"].isnull()]
    shipments_containers_df = containers_df[~containers_df["shipment_id"].isnull()]
    return propositions_containers_df, shipments_containers_df


def merge_shipments_with_containers(
    shipments_df: pd.DataFrame, containers_df: pd.DataFrame, drop_duplicate_key="shipment_id",
):
    (propositions_containers_df, shipments_containers_df,) = split_propositions_shipments_containers(containers_df)
    shipments_df = merge_with_shipments_containers_then_with_propositions_containers(
        shipments_df=shipments_df,
        shipments_containers_df=shipments_containers_df,
        propositions_containers_df=propositions_containers_df,
    )
    shipments_df = shipments_df.sort_values(by=["teus", "tc"], ascending=False)
    shipments_df = shipments_df[
        (~shipments_df.duplicated(subset=[drop_duplicate_key])) | (shipments_df[drop_duplicate_key].isnull())
    ]  # drop duplicates without dropping nan
    return shipments_df



import pandas as pd


def calculate_turnover(prices_df: pd.DataFrame):
    categories_to_sum = [
        "departure_truck_freight",
        "departure_fees",
        "freight",
        "arrival_fees",
        "arrival_truck_freight",
        "insurance",
    ]
    prices_df_filtered = prices_df[prices_df["category"].isin(categories_to_sum)]

    # the turnover calculation uses purchase and margin prices coming from the final_check (or from the proposition when
    # at least one purchase or one margin price is missing from the final_check)
    prices_df_filtered.loc[:, "turnover"] = prices_df_filtered.loc[:, "purchase_price_in_eur"
                                            ] + prices_df_filtered.loc[:, "margin_price_in_eur"]

    # the initial turnover calculation uses initial purchase and initial margin prices coming from the proposition
    prices_df_filtered.loc[:, "turnover_initial"] = prices_df_filtered.loc[:, "initial_purchase_price_in_eur"
                                                    ] + prices_df_filtered.loc[:, "initial_margin_price_in_eur"]

    return prices_df_filtered.groupby("shipment_id", as_index=False).agg({"turnover": sum, "turnover_initial": sum})
